﻿using System;
using Ozeki.Network;
using Ozeki.VoIP;

namespace PBX_Database
{
    class MyPBX : PBXBase
    {
        string localAddress;
        DatabaseManager databaseManager;

        public MyPBX(string localAddress, int minPortRange, int maxPortRange)
            : base(minPortRange, maxPortRange)
        {
            this.localAddress = localAddress;
            Console.WriteLine("PBX starting...");
            Console.WriteLine("Local address: " + localAddress);

            databaseManager = new DatabaseManager();
        }

        protected override void OnStart()
        {
            Console.WriteLine("PBX started.");
            SetListenPort(localAddress, 5060, TransportType.Udp);

            Console.WriteLine("Listened port: 5060(UDP)");

            SetUserInfosDB();
            base.OnStart();
        }

        protected override AuthenticationResult OnAuthenticationRequest(ISIPExtension extension, RequestAuthenticationInfo authInfo)
        {
            Console.WriteLine("Authentication request received from: " + extension.ExtensionID);

            AuthenticationResult result = new AuthenticationResult();

            UserInfo userInfo;
            if (databaseManager.TryGetUserInfo(extension.ExtensionID))
            {
                userInfo = databaseManager.GetUserInfo(extension.ExtensionID);
                result = extension.CheckPassword(userInfo.UserName, userInfo.Password, authInfo);
            }
            else
            {
                Console.WriteLine("Cannot find extension. UserName: " + extension.ExtensionID);
            }

            if (result.AuthenticationAccepted)
            {
                Console.WriteLine("Authentication accepted. UserName: " + extension.ExtensionID);
            }
            else
            {
                Console.WriteLine("Authentication denied. UserName: " + extension.ExtensionID);
            }

            return result;
        }

        protected override RegisterResult OnRegisterReceived(ISIPExtension extension, SIPAddress from, int expires)
        {
            Console.WriteLine("Register received from: " + extension.ExtensionID);
            return base.OnRegisterReceived(extension, from, expires);
        }

        protected override void OnUnregisterReceived(ISIPExtension extension)
        {
            Console.WriteLine("Unregister received from: " + extension.ExtensionID);
            base.OnUnregisterReceived(extension);
        }

        protected override void OnCallRequestReceived(ISessionCall call)
        {
            Console.WriteLine("Call request received. Caller: " + call.DialInfo.CallerID + " callee: " + call.DialInfo.Dialed);
            base.OnCallRequestReceived(call);
        }

        void SetUserInfosDB()
        {
            for (int i = 0; i < 10; i++)
            {
                var userName = string.Format("100{0}", i);
                var password = userName;
                UserInfo userInfo = new UserInfo(userName, password);
                databaseManager.AddUserInfo(userInfo);
            }
        }

    }
}
